<?php

class ExpayOrder
{
    protected $context;
    protected $module;

    public function __construct()
    {
        $this->context = Context::getContext();
        $this->module = Module::getInstanceByName('expay');
    }

    public function getCustomerSecureKey()
    {
        $customer = new Customer($this->context->cart->id_customer);

        return $customer->secure_key;
    }

    public function getCartOrderTotal()
    {
        return (float)$this->context->cart->getOrderTotal();
    }

    public function getCartId()
    {
        return (int)$this->context->cart->id;
    }

    public function getCurrencyId()
    {
        return (int)$this->context->currency->id;
    }

    /**
     * The optional message that will be used to attach to the order.
     *
     * @return string
     */
    public function getOptionalMessage()
    {
        return null;
    }

    /**
     * Return the array of extra variables.
     *
     * The extra variables will be used to save to database and attach to the order email.
     *
     * @param string $id_charge The Expay charge ID.
     *
     * @return array
     */
    public function getExtraVariables($id_charge)
    {
        $extra_variables = [];

        if (!empty($id_charge)) {
            $extra_variables['transaction_id'] = $id_charge;
        }

        return $extra_variables;
    }

    /**
     * The flag that used to indicate that the PrestaShop need to
     * round the card order total amount.
     *
     * If the flag is false, the PrestaShop will perform rounding.
     * If the flag is true, the PrestaShop WILL NOT preform rounding.
     *
     * @return bool
     */
    public function isNotNeededRoundingCardOrderTotal()
    {
        return false;
    }

    public function save($id_order_state, $payment_method, $id_charge = null)
    {
        $this->module->validateOrder(
            $this->getCartId(),
            $id_order_state,
            $this->getCartOrderTotal(),
            $payment_method,
            $this->getOptionalMessage(),
            $this->getExtraVariables($id_charge),
            $this->getCurrencyId(),
            $this->isNotNeededRoundingCardOrderTotal(),
            $this->getCustomerSecureKey()
        );
    }

    /**
     * The successful order status.
     *
     * @return int
     */
    public function getOrderStateAcceptedPayment()
    {
        return Configuration::get('PS_OS_PAYMENT');
    }

    /**
     * The order status that indicate the order is canceled.
     *
     * @return int
     */
    public function getOrderStateCanceled()
    {
        return Configuration::get('PS_OS_CANCELED');
    }

    /**
     * The order status that indicate the order is in progress.
     *
     * @return int
     */
    public function getOrderStateProcessingInProgress()
    {
        return Configuration::get('PS_OS_PREPARATION');
    }

    /**
     * Update an order payment transaction ID to database.
     *
     * Note:
     * - The PrestaShop order payment transaction ID has been mapped with Expay charge ID.
     * - PrestaShop has order and order payment separate from each other. So, to update an order payment transaction ID,
     * it need to retrieve the order payment from the order.
     *
     * @param int $id_order
     * @param string $transaction_id The order payment transaction ID. It is the reference ID between PrestaShop order
     * payment and Expay payment gateway.
     */
    public function updatePaymentTransactionId($id_order, $transaction_id)
    {
        $order = new Order($id_order);
        $order_payment_collection = $order->getOrderPaymentCollection();

        $order_payment = $order_payment_collection[0];

        $order_payment->transaction_id = $transaction_id;
        $order_payment->update();
    }

    /**
     * @param \Order $order The instance of class, Order.
     */
    public function updateStateToBeCanceled($order)
    {
        $order_state = $this->getOrderStateCanceled();

        if ($order->current_state == $order_state) {
            return;
        }

        $order->setCurrentState($order_state);
    }

    /**
     * @param \Order $order The instance of class, Order.
     */
    public function updateStateToBeSuccess($order)
    {
        $order_state = $this->getOrderStateAcceptedPayment();

        if ($order->current_state == $order_state) {
            return;
        }

        $order->setCurrentState($order_state);
    }
}