<?php
if (defined('_PS_MODULE_DIR_')) {
    require_once _PS_MODULE_DIR_ . 'expay/provider/Provider.php';
    require_once _PS_MODULE_DIR_ . 'expay/classes/expay_order.php';
    require_once _PS_MODULE_DIR_ . 'expay/classes/expay_logger.php';
}

/** The class that initiates the payment and, depending on the response of the Expay server, generating templates */
class ExpayPaymentModuleFrontController extends ModuleFrontController
{
    private $redirect_success;
    private $redirect_reject;
    private $redirect_waiting;

    private $expay_order;
    private $provider;
    private $logger;

    public function __construct()
    {
        parent::__construct();

        $this->expay_order = new ExpayOrder();
        $this->provider = new ExpayProvider();
        $this->logger = new ExpayLogger();
    }

    private function checkCart($cart)
    {
        if ($cart->id_customer == 0 || $cart->id_address_delivery == 0 || $cart->id_address_invoice == 0 || !$this->module->active) {
            Tools::redirect('index.php?controller=order&step=1');
        }

        // Check that this payment option is still available in case the customer changed his address just before the end of the checkout process
        $authorized = false;
        foreach (Module::getPaymentModules() as $module) {
            if ($module['name'] == 'expay') {
                $authorized = true;
                break;
            }
        }
        if (!$authorized) {
            die($this->module->l('This payment method is not available.', 'payment'));
        }
    }

    private function generateSuccessRedirectUrl()
    {
        return 'index.php?controller=order-confirmation' .
            '&id_cart=' . $this->context->cart->id .
            '&id_module=' . $this->module->id .
            '&id_order=' . $this->module->currentOrder .
            '&key=' . $this->context->customer->secure_key;
    }

    private function generateWaitingRedirectUrl($order_id)
    {
        return 'index.php?controller=order-detail' . '&id_order=' . $order_id;
    }

    public function postProcess()
    {
        parent::postProcess();

        if (!isset($_POST['paymentMethod'])) {
            $this->logger->add('Expay: invalid request.');
            return false;
        }

        $cart = $this->context->cart;
        $this->checkCart($cart);

        $attributes = [];
        $methodList = $this->provider->getMethodList();
        foreach ($methodList as $method) {
            if ($method['alias'] === $_POST['paymentMethod']) {
                $service_id = $method['id'];
                $payment_option_name = $method['name'];
                if (count($method['attributes']) != 0) {
                    foreach ($method['attributes'] as $attr) {
                        if (!empty($_POST[$attr['key']])) {
                            $attributes[$attr['key']] = $_POST[$attr['key']];
                        }
                    }
                }
                break;
            }
        }

        $this->expay_order->save(Configuration::get('PS_OS_PREPARATION'), $payment_option_name);

        $order = Order::getByCartId($this->context->cart->id);
        $amount = sprintf("%.2f", $this->context->cart->getOrderTotal(true, Cart::BOTH));

        $this->redirect_success = $this->generateSuccessRedirectUrl();
        $this->redirect_waiting = $this->redirect_reject = $this->generateWaitingRedirectUrl($order->id);

        $response = [];
        $response = $this->provider->initPayment(
            $service_id,
            $amount,
            $order->id,
            $this->redirect_success,
            $this->redirect_waiting,
            $this->redirect_reject,
            $attributes
        );

        if (is_null($response)) {
            $this->expay_order->updateStateToBeCanceled($order);

            $message = "Expay Error: Response from Expay server is empty";
            $this->logger->add($message);

            $this->context->smarty->assign('message', "Internal Server Error. Order id: $order->id");
            $this->setTemplate('module:expay/views/templates/front/payment_error.tpl');
            return;
        }

        if (isset($response['error'])) { // ERROR
            $this->expay_order->updateStateToBeCanceled($order);

            $this->logger->add('Expay Error' . $response['error']['message'] .
                '. Order id: ' . $order->id);

            $this->context->smarty->assign('message', "Internal Server Error. Order id: $order->id");
            $this->setTemplate('module:expay/views/templates/front/payment_error.tpl');
            return;
        }

        if ($response['status'] == 204) { // state of payment - canceled
            $this->expay_order->updateStateToBeCanceled($order);

            $message = "Expay Error: Payment status is " . $response['status'] . "Order id: $order->id";
            $this->logger->add($message);

            $this->context->smarty->assign('message', "Payment is rejected. Order id: $order->id");
            $this->setTemplate('module:expay/views/templates/front/payment_error.tpl');
            return;
        }

        if (count($response['attributes']) != 0) {
            foreach ($response['attributes'] as $attr) {
                switch ($attr['key']) {
                    case 'redirectUrl':
                        $this->setRedirectAfter($attr['value']);
                        return;

                    case 'qrcode':
                        $this->context->smarty->assign('src', $attr['value']);
                        $this->setTemplate('module:expay/views/templates/front/qr.tpl');
                        return;

                    case 'receiptUrl': // PDF
                        $this->context->smarty->assign('src', $attr['value']);
                        $this->setTemplate('module:expay/views/templates/front/download_pdf.tpl');
                        return;

                    default:
                        continue;
                }
            }
        } else { // OFFLINE
            $this->setRedirectAfter($this->redirect_waiting);
            return;
        }
    }
}