<?php
if (defined('_PS_CONFIG_DIR_')) {
    require_once _PS_CONFIG_DIR_ . 'config.inc.php';
}

if (defined('_PS_MODULE_DIR_')) {
    require_once _PS_MODULE_DIR_ . '/expay/classes/expay_logger.php';
    require_once _PS_MODULE_DIR_ . '/expay/classes/expay_order.php';
}

/** Class Base Event for all Expay events */
abstract class ExpayBaseEvent
{
    protected $expay_logger;
    protected $payment_order;
    protected $provider;

    public function __construct()
    {
        $this->expay_logger = new ExpayLogger();
        $this->payment_order = new ExpayOrder();
        $this->provider = new ExpayProvider();
    }

    public abstract function handle($event);

    protected function sendResponse($code, $message)
    {
        $response = [
            'response' => [
                'status' => $code,
                'message' => $message,
                'timestamp' => time()
            ]
        ];
        $response['hash'] = $this->provider->hashResponse($response);
        header('Content-type: application/json');
        exit(json_encode($response));
    }


    /**
     * Check that the order exists in prestashop
     *
     * @return bool
     */
    protected function isOrderValid($order)
    {
        if ($isInvalid = !Validate::isLoadedObject($order)) {
            $this->expay_logger->add('Expay Webhooks: the ' . $order->id . 'doesn\'t exist');
        }

        return !$isInvalid;
    }

    /**
     * Check that the order status is 'Canceled' in prestashop
     *
     * @return bool
     */
    protected function isOrderCanceled($order)
    {
        if ($isCanceled = ($order->current_state === Configuration::get('PS_OS_CANCELED'))) {
            $this->expay_logger->add('Expay Webhooks: the order#' . $order->id . ' has already been canceled');
        }

        return $isCanceled;
    }

    /**
     * Check that the order status is 'Accepted' in prestashop
     *
     * @return bool
     */
    protected function isOrderPaid($order)
    {
        if ($isPaid = ($order->current_state === Configuration::get('PS_OS_PAYMENT'))) {
            $this->expay_logger->add('Expay Webhooks: the order#' . $order->id . ' has already been paid');

        }

        return $isPaid;
    }

    /**
     * Check that the hash is valid
     *
     * @return bool
     */
    protected function isHashValid($event)
    {
        if (!isset($event['hash'])) {
            $this->expay_logger->add('Expay Webhooks: invalid hash');

            return false;
        }

        $hash = $event['hash'];
        unset($event['hash']);
        if ($hash !== $this->provider->hashResponse($event)) {
            $this->expay_logger->add('Expay Webhooks: invalid hash');

            return false;
        }

        return true;
    }

    /**
     * Check that the Prestashop order amount matсhes amount from Expay Server
     *
     * @return bool
     */
    protected function amountMatching($order, $event)
    {
        if ($amountNotMatches = ($order->total_paid != $event['amount'])) {
            $this->expay_logger->add('Expay Webhooks: The amount of order ' . $event['order'] . " doesn't match.");
        }

        return !$amountNotMatches;
    }
}