<?php

use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

if (!defined('_PS_VERSION_')) {
    exit;
}

if (defined('_PS_MODULE_DIR_')) {
    require_once _PS_MODULE_DIR_ . 'expay/provider/Provider.php';
    require_once _PS_MODULE_DIR_ . 'expay/setting.php';
}

define('PS_PAYMENTMODULE_HOOKS', "displayOrderConfirmation,header,payment");

class Expay extends PaymentModule
{
    public $provider;
    public $categories_list;
    public $available_methods;
    public $img_list;
    protected $setting;

    public function __construct()
    {
        $this->setting = new ExpaySetting();
        $this->provider = new ExpayProvider();
        $this->name = 'expay';
        $this->tab = 'payments_gateways';
        $this->version = '1.0.1';
        $this->ps_versions_compliancy = ['min' => '1.7', 'max' => _PS_VERSION_];
        $this->author = 'elementpay';
        $this->controllers = ['payment'];
        $this->is_eu_compatible = 1;

        $this->currencies = true;
        $this->currencies_mode = 'checkbox';

        $this->bootstrap = true;
        parent::__construct();

        $this->displayName = $this->l('Expay');
        $this->description = $this->l('Expay payment module');

        if (!count(Currency::checkPaymentCurrencies($this->id))) {
            $this->warning = $this->l('No currency has been set for this module.');
        }

        $this->categories_list = $this->provider->getServicesCategories();
        $this->available_methods = $this->provider->getMethodList();
        $this->img_list = $this->getImagesForAvailableMethods($this->available_methods);
    }

    public function install()
    {
        if (!parent::install() ||
            !$this->registerHook('Header') ||
            !$this->registerHook('paymentOptions') ||
            !$this->registerHook('paymentReturn')
        ) {
            return false;
        }
        return true;
    }

    /** func for settings render */
    public function getContent()
    {
        if ($this->setting->isSubmit()) {
            $this->setting->save();
            $this->smarty->assign('confirmation', $this->displayConfirmation($this->l('Settings updated')));
        }

        $this->smarty->assign([
            'live_public_key' => $this->setting->getLivePublicKey(),
            'live_secret_key' => $this->setting->getLiveSecretKey(),
            'module_status' => $this->setting->isModuleEnabled(),
            'sandbox_status' => $this->setting->isSandboxEnabled(),
            'submit_action' => $this->setting->getSubmitAction(),
            'test_public_key' => $this->setting->getTestPublicKey(),
            'test_secret_key' => $this->setting->getTestSecretKey(),
            'title' => $this->setting->getTitle(),
            'webhooks_endpoint' => $this->getWebhooksEndpoint(),
        ]);

        return $this->display(__FILE__, 'views/templates/admin/setting.tpl');
    }

    /** func for generation endpoint link */
    public function getWebhooksEndpoint()
    {
        return $this->context->link->getModuleLink('expay', 'webhooks', [], true);
    }

    public function hookDisplayOrderConfirmation($params)
    {
        if (!$this->active) {
            return;
        }

        if ($params['order']->module != $this->name) {
            return;
        }

        $this->smarty->assign('order_reference', $params['order']->reference);

        return $this->smarty->fetch('module:expay/views/templates/front/payment_infos.tpl');
    }

    public function hookHeader()
    {
        $this->context->controller->addJS($this->_path . 'js/payment_option.js', true);
        $this->context->controller->addCSS($this->_path . 'css/expay-payment.css', 'all');
        $this->context->controller->addJquery();
        $this->context->controller->addJqueryPlugin('fancybox');
    }

    /**
     * Display payment options
     *
     * @return array of available payment options (Payment Option = Expay Service Category)
     */
    public function hookPaymentOptions($params)
    {
        if (!$this->active) {
            return;
        }

        if (!$this->checkCurrency($params['cart'])) {
            return;
        }

        return $this->getPaymentOptions();
    }

    /**
     * Apply passed callable function to all hooks for the module, bailing if any function fails
     *
     * @return bool
     */
    protected function applyToHooks($callable)
    {
        foreach (explode(',', PS_PAYMENTMODULE_HOOKS) as $hook) {
            if (!($res = call_user_func($callable, $hook))) {
                break;
            }
        }
        return $res;
    }

    public function checkCurrency($cart)
    {
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);

        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }
        return false;
    }

    public function getPaymentOptions()
    {
        if (!$this->setting->isModuleEnabled() || $this->available_methods == false) {
            return null;
        }

        $options = [];
        foreach (['card', 'internet_banking', 'atm', 'mobile_banking', 'electronic_money', 'other'] as $method) {
            $methodName = implode(array_map('ucfirst', explode('_', $method)), '');
            if ($this->isPaymentEnabled($methodName)) {
                $options[] = $this->{'generate' . $methodName . 'PaymentOption'}();
            }
        }

        return $options;
    }

    private function isPaymentEnabled($key)
    {
        if (is_null($this->provider)) {
            return false;
        }
        if (isset($this->categories_list[$key])) {
            if ($this->categories_list[$key]['service_id'] == '' && count($this->categories_list[$key]['services']) == 0) {
                return false;
            }
        }

        return true;
    }

    private function getDataOfCategory($key)
    {
        return $this->categories_list[$key];
    }

    private function getImagesForAvailableMethods($available_methods)
    {
        if ($available_methods == false) {
            return false;
        }
        $img_list = [];
        $default_img_path = '/modules/expay/img/default.jpg';

        foreach ($available_methods as $method) {
            $file_path = '/modules/expay/img/' . $method['alias'] . '.png';
            $absolute_path = __DIR__ . '/img/' . $method['alias'] . '.png';
            if (file_exists($absolute_path)) {
                $img_list[$method['alias']] = $file_path;
            } elseif (!empty($method['image'])) {
                $img_list[$method['alias']] = $method['image'];
            } else {
                $img_list[$method['alias']] = $default_img_path;
            }
        }
        return $img_list;
    }

    private function displayPaymentOption($category_data, $template_name)
    {
        $methods_of_category = [];

        foreach ($this->available_methods as $method) {
            if (in_array($method['id'], $category_data['services'])) {
                array_push($methods_of_category, $method);
            }
        }

        $this->smarty->assign([
            'Category' => $category_data,
            'MethodList' => $methods_of_category,
            'imgList' => $this->img_list,
            'js_path' => $this->_path . 'js/'
        ]);
        return $this->display(__FILE__, $template_name);
    }

    private function generateCardPaymentOption()
    {
        $category = $this->getDataOfCategory('card');

        $payment = new PaymentOption();
        $payment->setCallToActionText($category['name'])
            ->setModuleName('expay_' . $category['key'])
            ->setForm($this->displayPaymentOption($category, 'payment_option.tpl'));

        return $payment;
    }

    private function generateInternetBankingPaymentOption()
    {
        $category = $this->getDataOfCategory('internet_banking');

        $payment = new PaymentOption();
        $payment->setCallToActionText($category['name'])
            ->setModuleName('expay_' . $category['key'])
            ->setForm($this->displayPaymentOption($category, 'payment_option.tpl'));

        return $payment;
    }

    private function generateAtmPaymentOption()
    {
        $category = $this->getDataOfCategory('atm');

        $payment = new PaymentOption();
        $payment->setCallToActionText($category['name'])
            ->setModuleName('expay_' . $category['key'])
            ->setForm($this->displayPaymentOption($category, 'payment_option.tpl'));

        return $payment;
    }

    private function generateMobileBankingPaymentOption()
    {
        $category = $this->getDataOfCategory('mobile_banking');

        $payment = new PaymentOption();
        $payment->setCallToActionText($category['name'])
            ->setModuleName('expay_' . $category['key'])
            ->setForm($this->displayPaymentOption($category, 'payment_option.tpl'));

        return $payment;
    }

    private function generateElectronicMoneyPaymentOption()
    {
        $category = $this->getDataOfCategory('electronic_money');

        $payment = new PaymentOption();
        $payment->setCallToActionText($category['name'])
            ->setModuleName('expay_' . $category['key'])
            ->setForm($this->displayPaymentOption($category, 'payment_option.tpl'));

        return $payment;
    }

    private function generateOtherPaymentOption()
    {
        $category = $this->getDataOfCategory('other');

        $payment = new PaymentOption();
        $payment->setCallToActionText($category['name'])
            ->setModuleName('expay_' . $category['key'])
            ->setForm($this->displayPaymentOption($category, 'payment_option.tpl'));

        return $payment;
    }
}
