<?php
include __DIR__ . '/Curl.php';

require_once dirname(dirname(dirname(__DIR__))) . '/config/config.inc.php';
require_once dirname(__DIR__) . '/classes/expay_logger.php';
require_once dirname(__DIR__) . '/setting.php';

/** Expay class which provides all communication with Expay API */
class ExpayProvider
{
    const SANDBOX_URL = 'https://api.sandbox.expay.asia/merchant/';
    const LIVE_URL = 'https://api.expay.asia/merchant/';

    private $url;
    private $public_key;
    private $private_key;
    private $logger;

    public function __construct()
    {
        $this->logger = new ExpayLogger();
        $setting = new ExpaySetting();

        $this->public_key = $setting->getPublicKey();
        $this->private_key = $setting->getSecretKey();

        $this->url = $setting->isSandboxEnabled() ? self::SANDBOX_URL : self::LIVE_URL;
    }

    private function hashData($method, $params)
    {
        $string_for_hash = $method . http_build_query($params, null, '&', PHP_QUERY_RFC3986);
        return hash_hmac('sha1', $string_for_hash, $this->private_key);
    }

    public function hashResponse($response)
    {
        return hash_hmac('sha1', http_build_query($response, null, '&', PHP_QUERY_RFC3986), $this->private_key);
    }

    public function isResponseValid($response)
    {
        if (isset($result['error']) ||
            isset($result['body']['error']) ||
            !empty($result['curl_error']) ||
            $result['http_code'] === 502
        ) {
            $message = 'Expay Provider: Error response of getMethodList';
            $this->logger->add($message);

            return false;
        }

        return true;
    }

    public function getMethodList()
    {
        $cache_key = 'expay-getMethodList';
        $cache = Cache::getInstance();
        if ($cache->exists($cache_key)) {
            return $cache->get($cache_key);
        }

        $getMethodsURL = $this->url . 'getMethods';
        $params = [
            'key' => $this->public_key,
            'timestamp' => time(),
        ];
        $params['hash'] = $this->hashData('getMethods?', $params);

        $ch = new Curl();
        $result = $ch->post($getMethodsURL, $params);

        if (isset($result['error']) ||
            !empty($result['curl_error']) ||
            isset($result['body']['error']) ||
            $result['http_code'] === 502
        ) {
            $message = 'Expay Provider: Error response of getMethodList';
            $this->logger->add($message);

            return false;
        }
        $cache->set($cache_key, $result['body']['response']['methods']);

        return $result['body']['response']['methods'];
    }

    public function getServicesCategories()
    {
        $cache_key = 'expay-getServicesCategories';
        $cache = Cache::getInstance();
        if ($cache->exists($cache_key)) {
            return $cache->get($cache_key);
        }

        $getServicesURL = $this->url . "getCategories";

        $params = [
            'key' => $this->public_key,
            'timestamp' => time(),
        ];
        $params['hash'] = $this->hashData('getCategories?', $params);

        $ch = new Curl();
        $result = $ch->post($getServicesURL, $params);

        if (isset($result['error']) || !empty($result['curl_error']) || isset($result['body']['error']) || $result['http_code'] === 502) {
            $this->logger->add('Expay Provider: Error response of getServicesCategories');
            return false;
        }
        $cache->set($cache_key, $result['body']['response']['categories']);

        $categories = [];
        foreach ($result['body']['response']['categories'] as $category) {
            $categories[$category['key']] = $category;
        }

        return $categories;
        //return $result['body']['response']['categories'];
    }

    public function initPayment($service_id, $amount, $order, $url_success, $url_waiting, $url_reject, $attrs = [])
    {
        $initPaymentURL = $this->url . 'initPayment?';
        $ch = new Curl();

        $params = [
            'service_id' => $service_id,
            'amount' => $amount,
            'order' => $order,
            '_successUrl' => $url_success,
            '_waitingUrl' => $url_waiting,
            '_rejectUrl' => $url_reject,
            'key' => $this->public_key,
            'timestamp' => time()
        ];

        foreach ($attrs as $key => $value) {
            $params[$key] = $value;
        }

        $params['hash'] = $this->hashData('initPayment?', $params);

        $result = $ch->post($initPaymentURL, $params);
        if (isset($result['error']) || !empty($result['curl_error'])) {
            $this->logger->add('Expay Provider: Error response of initPayment');
            return false;
        }

        if (array_key_exists('error', $result['body'])) {
            $this->logger->add('Expay Provider: Error response of initPayment (' . $result['body']['error']['message'] . ').');

            return $result['body'];
        }

        return $result['body']['response'];
    }

    public function getPaymentStatus($payment_id)
    {
        $getStatusURL = $this->url . 'getStatus?';
        $params = [
            'payment_id' => $payment_id,
            'key' => $this->public_key,
            'timestamp' => time(),
        ];
        $params['hash'] = $this->hashData('getStatus?', $params);

        $ch = new Curl();
        $result = $ch->post($getStatusURL, $params);

        if (isset($result['error']) || !empty($result['curl_error'])) {
            $message = 'Expay Provider: Error response of getPaymentStatus';
            $this->logger->add($message);

            return false;
        }

        if (isset($result['body']['error'])) {
            $message = 'Expay Provider: Error response of getPaymentStatus (' . $result['body']['error']['message'] . ').';
            $this->logger->add($message);

            return false;
        }

        return $result['body']['response']['status'];
    }
}